%let version=2.0;
*********************************************************************************************************************************************;
*** ACS Linkage Program - Version 2.0                                                                                                        ;
/** SAS program to ACS variables based upon county and census tract information in registry incidence data.
    The program will generate 3 blocks of ACS data (2006-2010, 2010-2014, 2013-2017).
    
    THIS PROGRAM DOES NOT BLANK OUT CENSUS TRACT 2010!!!!  
    YOU SHOULD RUN THIS PROGRAM BEFORE RUNNING THE CENSUS TRACT POVERTY LINKAGE SAS PROGRAM - WHICH DOES BLANK OUT THE CENSUS TRACT(S).

    Name of file containing poverty group information by state, county, and census tract:
         yost-poverty-0610-1014-1317.dat

    NOTE: The following list identifies counties with possible geographic code errors or definition differences for tracts and block groups. 
    Counties in this list may generate an "Unknown" value as a result.
         Alabama: Bibb
         Illinois: Coles
         Iowa: Adair, Adams, Cass, Clay, Dickinson, Dubuque, Emmet, Page, Sioux, Union, Worth
         Maryland: Calvert
         Massachusetts: Berkshire
         Michigan: Baraga
         Minnesota: Murray
         Texas: Wilson

         *For more information please see http://www.census.gov/acs/www/data_documentation/geography_notes/#tracts

    Written for NAACCR, Inc.
    August 2019
**/
*********************************************************************************************************************************************;
**** Written/modified:                                                                                                                       ;
****    William Howe, Information Management Services, Inc., 301.680.9770                                                                    ;
*********************************************************************************************************************************************;

*********************************************************************************************************************************************;
/*** NAACCR Version 18.0 input registry file, with the following fields included:
       Patient ID [item #20]
       State at DX [item #80]
       County at DX for Analysis [item #89]
       Census Tract 2010 [item #135]
       Date of Diagnosis [item #390]
***/;

***filename regfile 'directory and file name';
filename regfile './sample.txt';

*** Output file Name - Text file, with Patient ID, state, county, census tract, census block, and generated Percent Poverty values;
*** filename outfile 'directory and file name' ;
filename outfile './out.txt';
*********************************************************************************************************************************************;

data _null_;
  currentYear = year(date());
  call symput('currentYear', currentYear);
run;

************************************************************************************************************************************************;
***** Read in the ACS data files ***************************************************************************************************************;
************************************************************************************************************************************************;
%let recl = 162;
%let len = %eval(&recl. - 12);
%let start = 3222;

data acs;
  infile "yost-poverty-0610-1014-1317.dat" lrecl=&recl.;
  input  @001 acs_key $char11.
         @012 acs_dat $char&len..;
run;
proc sort data = acs; by acs_key; run;

************************************************************************************************************************************************;
***** Read in data file, convert two letter state code to a 2 digit FIPS code, apply format to state/county/census tract, and write out records ;
************************************************************************************************************************************************;
data registry_data;
  infile regfile lrecl=4048 pad;
  
  input  @001 Incidence_record $char4048.
         @124 DataItem_80      $char2.
         @150 DataItem_89      $char3.
         @185 DataItem_135     $char6.
         @544 DataItem_390     $char8.;
         
  year_dx = input(substr(DataItem_390,1,4),4.);
  acs_key = DataItem_80||DataItem_89||DataItem_135;
run;
proc sort data = registry_data; by acs_key; run;

************************************************************************************************************************************************;
***** Merge the data ***************************************************************************************************************************;
************************************************************************************************************************************************;
data registry_data;
	merge registry_data (in=a) acs (in=b);
	by acs_key;
	if a;
run;

data registry_data;
	set registry_data;
  if not (2005 <= year_dx <= &currentYear.) then do;
     acs_dat = repeat(' ',&len.);
  end;
  
  if trim(acs_dat) = '' then flag = 0; else flag = 1;

  label Incidence_record = 'Incidence Record'
        DataItem_80      = 'Addr at DX - State'
        DataItem_89      = 'County at DX for Analysis'
        DataItem_135     = 'Census Tract 2010'
        DataItem_390     = 'Date of Diagnosis'
        acs_key          = 'State/County/Tract Lookup'
        acs_dat          = 'ACS Data'
        flag             = 'ACS Data Linkage Flag'
        year_dx          = 'Year of Diagnosis'; 
run;

/* Create a new file with ACS data */
data _null_;
  set registry_data;
  file outfile lrecl=4048 pad;
  put @001  Incidence_record $4048.
      @&start. acs_dat $char&len..;
  run;

****** Summarize Results;
proc format;
 value flag
       0= 'Not Linked'
       1= 'Linked';
 run;

proc freq data=registry_data;
  tables flag * year_dx / list missing;
  title "NAACCR - ACS Assignment Program version &version.";
  title2 'Frequency Distribution for ACS Linkage Flag';
  format flag flag.;
run;